'' =================================================================================================
''
''   File....... fc-4+_template__2015-09-01.spin
''   Purpose.... Template code for FC-4+ 
''   Author..... Jon McPhalen
''               Copyright (C) 2012-2015 EFX-TEK
''   E-mail..... teamefx@efx-tek.com
''   Started.... 
''   Updated.... 01 SEP 2015
''
'' =================================================================================================


con { speed & timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  PCB_REV = "C" 

  RX1     = 31  { I }                                            ' serial / programming / debug port
  TX1     = 30  { O }
  
  SDA     = 29  { I/O }                                          ' i2c / eeprom
  SCL     = 28  { I/O } 

  DMX_A0  = 27  { I }                                            ' DMX address                                                         
  DMX_A1  = 26  { I }                                                 
  DMX_A2  = 25  { I }
  DMX_A3  = 24  { I }                                           
  DMX_A4  = 23  { I }                                           
  DMX_A5  = 22  { I }                                           
  DMX_A6  = 21  { I }                                           
  DMX_A7  = 20  { I }
  DMX_A8  = 19  { I }

  AUX_0   = 18  { I/O } 
  AUX_1   = 17  { I/O }                                                         

  SIO_TX  = 16  { O }                                            ' { disabled by default }   
  SIO_RX  = 15  { I/O }                                          ' TTL serial

  R_LED   = 14  { O }                                            ' bi-color LED    
  G_LED   = 13  { O }                                            

  TX2     = 12  { O }                                            ' RS-485 interface  
  TX2_EN  = 11  { O }
  RX2_EN  = 10  { O }
  RX2     =  9  { I }
                                                         
  Z_CROSS =  8  { I }                                            ' zero-cross signal
                 
  OUT3    =  7  { O }                                            ' AC outputs
  OUT2    =  6  { O }
  OUT1    =  5  { O }
  OUT0    =  4  { O }
  
  OPT_SM  =  3  { I }                                            ' option inputs     
  OPT_A0  =  2  { I }                                              
  OPT_A1  =  1  { I }
  OPT_BR  =  0  { I }                                              
          

con

   #0, NO, YES
   
   #0, OFF, RED, GRN                                             ' bi-color LED states

   #0, NOCON, RX, TX, RXTX                                       ' rs-485 states

   LAST_CH    = 3                                                ' last available channel

   BAUD       = 38_400                                           ' baud rate for serial io
   
   RX_TIMEOUT =  25                                              ' rx timeout (ms)
   GLOBAL     = $FF                                              ' global address 


obj
                                                   
' main                                                           ' * master Spin cog                                         
  time  : "jm_time"                                              '   timing and delays                                
  prng  : "jm_prng"                                              '   random #s                               
  io    : "jm_io_basic"                                          '   essential IO                           
  lamps : "jm_ac_dimmer"                                         ' * AC fading (4 channels)                          
  fader : "jm_fade_manager"                                      ' * background fade management/control                       
  com1  : "jm_fullduplexserial"                                  ' * serial IO (SIO or terminal)                   
  dmxrx : "jm_dmx_rx"                                            ' * dmx input
  dmxtx : "jm_dmx_tx"                                            ' * dmx output
  rs485 : "jm_hd485"                                             ' * half duplex serial over RS-485                              

' * uses cog when started
  

pub main 

  setup

  repeat
    ' your app code here
    

pub setup

'' Setup objects and IO for typical FC-4+ app

  time.start                                                     ' setup timing/delays

  prng.seed(-cnt, -cnt ~> 4, $dead_beef, cnt << 4, cnt)          ' seed randomizer

  io.start($0000, $0000)                                         ' clear all

  lamps.start(OUT0, Z_CROSS)                                     ' start AC phase driver
' fader.start(4, lamps.address)                                  ' start fade controller

' com1.start(SIO_RX, SIO_RX, %1100, BAUD)                        ' start half-duplex serial (SIO)

' com1.start(RX1, TX1, %0000, BAUD)                              ' start full-duplex serial (terminal)


pub setup_dmx_in

'' Call this method if application is a DMX receiver

  set_rs485(RX)                                                  ' receive only
  dmxrx.start(RX2, G_LED, 1000)                                  ' start DMX rx, green LED = activity
  io.low(R_LED)                                                  ' required for green LED


pub setup_dmx_out

'' Call this method if application is a DMX transmitter

  set_rs485(NOCON)                                               ' release connections in this cog 
  dmxtx.start(TX2, TX2_EN)                                       ' start DMX tx


con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }


pub set_rgled(state)

'' Sets R/G LED

  case state
    RED   : outa[R_LED..G_LED] := %10
    GRN   : outa[R_LED..G_LED] := %01
    other : outa[R_LED..G_LED] := %00             

  dira[R_LED..G_LED] := %11


pub set_rs485(state)

'' Sets RS-485 RX and TX enable inputs

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11


pub ttl_inputs

'' Return AUX inputs status

  dira[AUX_1..AUX_0] := %00                                      ' force to input mode
  
  return ina[AUX_1..AUX_0]                                       ' return state


pub dmx_address

'' Return DMX address switch setting

  dira[DMX_A8..DMX_A0] := %0_0000_0000                           ' force to input mode

  return ina[DMX_A8..DMX_A0]                                     ' return states


pub option_inputs

'' Returns Option switch inputs as active-high
'' -- on = 1

  dira[OPT_BR..OPT_SM] := %0000

  return !ina[OPT_BR..OPT_SM] & %1111
  

pub ucase(c)

'' Convert c to uppercase
'' -- does not modify non-alphas

  if ((c => "a") & (c =< "z"))                                   ' lower case now?
    c -= $20                                                     '   yes, convert to upper

  return c


dat

{{

  Copyright (C) 2012-2015 EFX-TEK

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      